/*
    Copyright (C) 2003 Fons Adriaensen <fons.adriaensen@skynet.be>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <clxclient.h>


#define X0  8
#define DX  28


static char **cmpptr;

extern "C" int compare (const void *a, const void *b)
{
    return strcmp (cmpptr [*(int *)a], cmpptr [*(int *)b]); 
}


X_browse::X_browse (X_window  *parent, X_callback *callb, X_browse_style *style,
                    int xp, int yp, int xs, int ys,
                    int max_item, int max_char) :
    X_window (parent, xp, yp, xs, ys, style->bg), 
    _callb (callb),
    _style (style),
    _xs (xs),
    _ys (ys),
    _max_item (max_item),
    _max_char (max_char)
{
    _nrow = ys / style->dy;
    _nclm = (_max_item + _nrow - 1) / _nrow;
    _buff = new char [max_char];    
    _ptr = new char* [max_item];    
    _len = new short [max_item];    
    _col = new short [max_item];    
    _ind = new int [max_item];    
    _dxc = new int [_nclm];
    _nclm = 0;
    reset ();
    x_add_events (ExposureMask | ButtonPressMask);
}


X_browse::~X_browse (void)
{
    delete []_buff; 
    delete []_ptr; 
    delete []_len; 
    delete []_col; 
    delete []_ind;
    delete []_dxc; 
}


void X_browse::handle_event (XEvent *E)
{
    switch (E->type)
    {
    case Expose:
	expose ((XExposeEvent *) E);
	break;  

    case ButtonPress:
	bpress ((XButtonEvent *) E);
	break;  
    }
}


void X_browse::expose (XExposeEvent *E)
{
    if (_nclm && E->count == 0) update (false);
}


void X_browse::bpress (XButtonEvent *E)
{
    int d, x, y, r, c;

    if (! _nclm) return;
    x = E->x - X0;
    y = E->y;
    d = _style->dy;
    r = y / d;
    y -= r * d;
    if ((y < 2) || (y > d - 2)) return;

    c = _clm0;
    do
    {
        d = _dxc [c];
	if ((x > 0) && (x < d))
	{
	    _sel = _ind [c * _nrow + r]; 
            _callb->handle_callb (X_callback::BROWSE_SEL, this, (XEvent *) E);    
            return;
	}
        x -= d + DX;
    }
    while (c++ < _clm1);
}


int X_browse::add_item (const char *txt, int col, int len)
{
    if (! len) len = strlen (txt);
    if (! len) return 0;
    if (_n_item == _max_item) return 1; 
    if (_n_char + len + 1 > _max_char) return 1;
    _ptr [_n_item] = _buff + _n_char;
    _len [_n_item] = len;
    _col [_n_item] = col;
    _ind [_n_item] = _n_item;
    strcpy (_buff + _n_char, txt);
    _n_char += len + 1;
    _n_item += 1;
    return 0;
}


void X_browse::sort (void)
{
    cmpptr = _ptr;
    qsort (_ind, _n_item, sizeof (int), compare);
}


void X_browse::show (void)
{
    _nclm = (_n_item + _nrow - 1) / _nrow;
    for (int i = 0; i < _nclm; i++) _dxc [i] = 0;
    _clm0 = 0;
    update (true);
}


void X_browse::forw (void)
{
    if (_clm0 == _clm1)
    {
	if (_clm0 < _nclm - 1)
	{
	    _clm0++;
	    update (true);
	}
    }
    else if (_clm1 != _nclm)
    {
	_clm0 = _clm1;
        update (true);
    }
}


void X_browse::back (void)
{
    int x, d;

    if (_clm0)
    {
	x = X0 + _dxc [--_clm0] + DX;
        while (_clm0 && (x + (d = _dxc [_clm0 - 1]) < _xs)) 
	{
	    x += d + DX;
            _clm0--;
	}
        update (true);
    }
}


void X_browse::update (bool clear)
{
    int  i, j, k, c, d, t;
    int  x, y, y0;
    bool f;
    GC gc = disp ()->dgc ();
 
    XSetFont (dpy (), gc, _style->font->fid);
    XSetFunction (dpy (), gc, GXcopy);
    if (clear) XClearWindow (dpy (), win ());

    y0 = (_style->dy + _style->font->ascent - _style->font->descent) / 2;

    x = X0;
    c = _clm0;

    while ((c < _nclm) && (x <= _xs - 4))
    {
        d = _dxc [c];
        f = (d == 0); 
        y = y0;        
        for (i = 0, j = c * _nrow; (i < _nrow) && (j < _n_item); i++, j++)
	{
	    k = _ind [j];
            if (f)
	    {     
                t = XTextWidth (_style->font, _ptr [k], _len [k]);             
                if (d < t) d = t;
	    }
            XSetForeground (dpy (), gc, _style->col [_col [k] & 3]);
            XDrawString (dpy (), win (), gc, x, y, _ptr [k], _len [k]);
            y += _style->dy;
	}
        if (f) _dxc [c] = d;    
        x += d;
        if (x <= _xs) c++;
        x += DX;
    }
    _clm1 = c;
}

