/*
    Copyright (C) 2003 Fons Adriaensen <fons.adriaensen@skynet.be>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <clxclient.h>


X_button::X_button (X_window       *parent,
                    X_callback     *callb,
                    X_button_style *style,
                    int xp, int yp,
                    const char *txt1,
                    const char *txt2,
                    int  cbid) :
    _callb (callb),
    _style (style),
    _xs (style->size.x),
    _ys (style->size.y),
    _cbid (cbid),
    _down (0),
    _stat (0),
    _col0 (style->color.col0),
    _col1 (style->color.col1),
    _text (style->color.text), 
    _len1 (0),
    _len2 (0),
    X_window (parent, xp, yp, style->size.x, style->size.y, style->color.shadow.parc)
{
    x_add_events (ExposureMask);
    if (_callb) x_add_events (ButtonPressMask | ButtonReleaseMask);
    set_text (txt1, txt2);
}


X_button::~X_button (void)
{
}


void X_button::handle_event (XEvent *E)
{
    switch (E->type)
    {
    case Expose:
	expose ((XExposeEvent *) E);
	break;  
 
    case ButtonPress:
	bpress ((XButtonEvent *) E);
	break;  

    case ButtonRelease:
	brelease ((XButtonEvent *) E);
	break;
    }
}


void X_button::bpress (XButtonEvent *E)
{
    _down = 1;
    redraw ();
    _callb->handle_callb (X_callback::BUTTON_DN, this, (XEvent *) E);
}


void X_button::brelease (XButtonEvent *E)
{
    _down = 0;
    redraw ();
    _callb->handle_callb (X_callback::BUTTON_UP, this, (XEvent *) E);
}


void X_button::expose (XExposeEvent *E)
{
    if (E->count == 0) redraw ();
}


void X_button::set_text (const char *txt1, const char *txt2)
{
    _len1 = _len2 = 0;
    if (txt1)
    {
	while ((*txt1 >= ' ') && (_len1 < MAXLEN)) _txt1 [_len1++] = *txt1++;
    }
    if (txt2)
    {
	while ((*txt2 >= ' ') && (_len2 < MAXLEN)) _txt2 [_len2++] = *txt2++;
    }
}


void X_button::set_stat (bool stat)
{
    if (_stat != stat)
    {
	_stat = stat;
        redraw ();
    }
}


void X_button::redraw (void)
{
    int  x, y, k, a, d;
    GC gc = disp ()->dgc ();

    XClearWindow (dpy (), win ());
    XSetFunction (dpy (), gc, GXcopy);

    XSetForeground (dpy (), gc, ((_style->type & X_button_style::LED) || ! _stat) ? _col0 : _col1);
    XFillRectangle (dpy (), win (), gc, 1, 1, _xs - 2, _ys - 2);

    XSetForeground (dpy (), gc, _text);
    XSetFont (dpy (), gc, _style->font->fid);
    a = _style->font->ascent;
    d = _style->font->descent;
    if (_len1)
    {
	x = (_style->type & X_button_style::LED) ? 11 : 0; 
	if (_len2) y = _ys / 2 - d;
	else       y  = (_ys + a - d) / 2;
	k = XTextWidth (_style->font, _txt1, _len1);
	XDrawString (dpy (), win (), gc, (_xs + x - k) / 2, y, _txt1, _len1); 
    }
    if (_len2)
    {
	if (_len1) y = _ys / 2 + a;
	else       y  = (_ys + a - d) / 2;
	k = XTextWidth (_style->font, _txt2, _len2);
	XDrawString (dpy (), win (), gc, (_xs - k) / 2, y, _txt2, _len2); 
    }

    if ((_style->type & X_button_style::LED) && _stat)
    {
	XSetForeground (dpy (), gc, _col1);
	XFillRectangle (dpy (), win (), gc, 2, 2, 9, 9);
    }

    XSetLineAttributes (dpy (), gc, 1, LineSolid, CapButt, JoinBevel);
    switch (_style->type & 255)
    {
    case X_button_style::BORDER:
	XSetForeground (dpy (), gc, _text);
	XDrawRectangle (dpy (), win (), gc, 0, 0, _xs - 1, _ys - 1);
	break;

    case X_button_style::RAISED:
	XSetForeground (dpy (), gc, _down ? _style->color.shadow.dark : _style->color.shadow.lite);
	XDrawLine (dpy (), win (), gc,  0, 0, 0, _ys - 1);
	XDrawLine (dpy (), win (), gc,  0, 0, _xs -1,  0);
	XSetForeground (dpy (), gc, _down ? _style->color.shadow.lite : _style->color.shadow.dark);
	XDrawLine (dpy (), win (), gc, _xs - 1, _ys - 1, _xs - 1, 1);
	XDrawLine (dpy (), win (), gc, _xs - 1, _ys - 1, 1, _ys - 1);
	break;
    }
}

