/*
    Copyright (C) 2003 Fons Adriaensen <fons.adriaensen@skynet.be>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <clxclient.h>
#include <X11/keysym.h>


#define DX 8


X_enumip::X_enumip (X_window *parent, 
                    X_callback      *callb,
                    X_textln_style  *style,
                    int xp, int yp,
                    int xs, int ys,
                    X_enip_item  *list,
                    X_linked     *back,
                    X_linked     *forw) :
    X_window (parent, xp, yp, xs, ys, style->color.normal.bgnd),
    X_linked (back, forw),
    _style (style),
    _callb (callb), 
    _list (list),
    _ind (0), _len (0), _foc (0), _txt (0),
    _xs (xs), _ys (ys), _x0 (xs - DX)
{
    int i, l, m;

    x_add_events (ExposureMask | ButtonPressMask | FocusChangeMask);
    _y0 = (_ys + style->font->ascent - style->font->descent) / 2;

    if (_list)
    {
	for (i = m = 0; _list [i]._text; i++)
	{
	    l = strlen (list [i]._text);
	    if (l > m) m = l;
	}
	_max = i;
	_txt = new char [m];
    }
    else _max = 0;
}


X_enumip::~X_enumip (void)
{
    delete [] _txt;
}


void X_enumip::select (void)
{
    if (_callb) _callb->handle_callb (X_callback::ENUMIP_BUT, this, 0);
}


void X_enumip::handle_event (XEvent *xe)
{
    switch (xe->type)
    {
    case Expose:
	expose ((XExposeEvent *) xe);
	break;  
 
    case ButtonPress:
	bpress ((XButtonEvent *) xe); 
	break;  
 
    case KeyPress:
	keypress ((XKeyEvent *) xe);
	break;

    case FocusIn:
	setfocus ((XFocusChangeEvent *) xe);
	break;

    case FocusOut:
	remfocus ((XFocusChangeEvent *) xe);
	break;
    }
}


void X_enumip::expose (XExposeEvent *e)
{
    redraw ();
}


void X_enumip::bpress (XButtonEvent *e)
{
    if (e->button == Button2)
    {
	//    if (_foc) XConvertSelection (dpy (), XA_PRIMARY, XA_STRING, XInternAtom (dpy (),
	//                                 "XTIPSEL", False), win (), e->time);
    }
    else
    {
	select ();
    }
}


void X_enumip::keypress (XKeyEvent *e)
{
    int  k;
    char t [10];

    k = XLookupString (e, t, 10, 0, 0);
    if (k && (*t >= ' ') && (*t != 127)) {}
    else spkey (XLookupKeysym (e, 0));
}


void X_enumip::spkey (int k)
{
    switch (k)
    {
    case XK_Left:
    case 'b':
	if (--_ind < 0) _ind = _max - 1;
	set_ind (_ind);
	break;

    case XK_Right:
    case 'f':
	if (++_ind == _max) _ind = 0;
	set_ind (_ind);
	break;

    case XK_Up:
	if (_back) _back->select ();
	else cbkey (k);
	break;

    case XK_Return:
    case XK_Down:
    case XK_KP_Enter:
	if (_forw) _forw->select ();
	else cbkey (k);
	break;

    default:
	cbkey (k);
    }
}


void X_enumip::cbkey (int k)
{
    _key = k; 
    if (_callb) _callb->handle_callb (X_callback::ENUMIP_KEY, this, 0);
}


void X_enumip::redraw (void)
{
    GC gc = disp ()->dgc ();

    XClearWindow (dpy (), win ());
    XSetFunction (dpy (), gc, GXcopy);

    if (_len)
    {
	XSetFont (dpy (), gc, _style->font->fid);
	XSetForeground (dpy (), gc, _foc ? _style->color.focus.text : _style->color.normal.text);
	XDrawString (dpy (), win (), gc, _x0,  _y0, _txt, _len);
    }

    if (_callb)
    {
	XSetLineAttributes (dpy (), gc, 1, LineSolid, CapButt, JoinBevel);
	XSetForeground (dpy (), gc, _style->color.shadow.dark);
	XDrawLine (dpy (), win (), gc, 0, _ys - 2, 0, 0);
	XDrawLine (dpy (), win (), gc, 0, 0, _xs - 2, 0);
	XSetForeground (dpy (), gc, _style->color.shadow.lite);
	XDrawLine (dpy (), win (), gc, 1, _ys - 1, _xs - 1, _ys - 1);
	XDrawLine (dpy (), win (), gc, _xs - 1, _ys - 1, _xs - 1, 1);
    }
}


void X_enumip::set_ind (int ind)
{
    if ((ind < 0) || (ind >= _max))
    {
	_ind = -1;
	_len = 0;
    }
    else
    {
	strcpy (_txt, _list [_ind = ind]._text);
	_len = strlen (_txt);
    }
    _x0 = _xs - DX;
    if (_len) _x0 -= XTextWidth (_style->font, _txt, _len);
    redraw ();
}
   

void X_enumip::setfocus (XFocusChangeEvent *e)
{
    if (e->detail != NotifyPointer && ! _foc)
    {  
	_foc = 1;
	x_add_events (KeyPressMask);
	XSetWindowBackground (dpy (), win (),_style->color.focus.bgnd);
	redraw ();
    }
}


void X_enumip::remfocus (XFocusChangeEvent *e)
{
    if (_foc)
    {  
	_foc = 0;
	x_rem_events (KeyPressMask);
	XSetWindowBackground (dpy (), win (),_style->color.normal.bgnd);
	redraw ();
    }
}

