/*
    Copyright (C) 2003 Fons Adriaensen <fons.adriaensen@skynet.be>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <clxclient.h>


X_menuwin::X_menuwin (X_window       *parent,
                      X_callback      *callb,
                      X_menuwin_style *style,
                      int xp,
                      int yp,
		      X_menuwin_item  *items) :
    _callb (callb),
    _style (style),
    _items (items),
    _xs (10),
    _ys (10),
    _isel (-1),
    _open (0),
    X_window (parent, xp, yp, 100, 100, style->color.bgnd)
{
    int i, k, ys, dx, xm;
    X_menuwin_item *P;

    xm = 0; 
    ys = 1;  
    i = 0;
    P = items;
    while (i < MAXITEM)   
    {
	if (P->_bits & X_menuwin_item::SPACE) ys += _style->step / 2;
	_ypos [i] = ys;
	_tlen [i] = k = strlen (P->_text);
	dx = XTextWidth (_style->font, P->_text, k);
	if (xm < dx) xm = dx;
	i++;
	ys += _style->step;
	if (P->_bits & X_menuwin_item::TITLE) ys++;
	if (P->_bits & X_menuwin_item::LAST) break;
	P++;
    }
     
    _nit = i;
    _xs = xm + DL + DR;
    _ys = ys + 2;

    if (_style->type & X_menuwin_style::PDOWN)
    {
	x_resize (_xs, _style->step);
	x_map ();
    }
    else x_resize (_xs, _ys);

    x_add_events (ExposureMask | ButtonPressMask | ButtonReleaseMask | PointerMotionMask | EnterWindowMask | LeaveWindowMask);
}


X_menuwin::~X_menuwin (void)
{
}


void X_menuwin::show (void)
{
    if (_style->type & X_menuwin_style::PDOWN) x_resize (_xs, _ys);
    x_mapraised ();
    _open = true;
    _isel = -1;
}


void X_menuwin::hide (void)
{
    if (_style->type & X_menuwin_style::PDOWN) x_resize (_xs, _style->step);
    else x_unmap ();
    _open = false;
}


void X_menuwin::handle_event (XEvent *E)
{
    switch (E->type)
    {
    case Expose:
	expose ((XExposeEvent *) E);
	break;  
 
    case ButtonPress:
	bpress ((XButtonEvent *) E);
	break;  

    case ButtonRelease:
	brelse ((XButtonEvent *) E);
	break;  

    case EnterNotify:
	enter ((XEnterWindowEvent *) E);
	break;

    case LeaveNotify:
	leave ((XLeaveWindowEvent *) E);
	break;

    case MotionNotify:
	motion ((XPointerMovedEvent *) E);
	break;
    }
}


void X_menuwin::bpress (XButtonEvent *E)
{
    if (_open)
    {
	if (_isel >= 0) _callb->handle_callb (X_callback::MENUWIN_SEL, this, (XEvent *) E);
	else            _callb->handle_callb (X_callback::MENUWIN_CAN, this, (XEvent *) E);
    }
    else _callb->handle_callb (X_callback::MENUWIN_OPEN, this, (XEvent *) E);
}


void X_menuwin::brelse (XButtonEvent *E)
{
    if (_open && (_isel >= 0)) _callb->handle_callb (X_callback::MENUWIN_SEL, this, (XEvent *) E); 
}


void X_menuwin::expose (XExposeEvent * E)
{
    if (E->count == 0) redraw (true);
}


void X_menuwin::enter (XEnterWindowEvent * E)
{
    if (_open) select (findit (E->x, E->y));
}


void X_menuwin::leave (XLeaveWindowEvent * E)
{
    if (_open) select (-1);
}


void X_menuwin::motion (XPointerMovedEvent * E)
{
    if (_open) select (findit (E->x, E->y));
}


void X_menuwin::redraw (bool clear)
{
    int  i, x, y, d;
    GC gc = disp ()->dgc ();
    const X_menuwin_item *P;

    if (clear) XClearWindow (dpy (), win ());

    XSetFunction (dpy (), gc, GXcopy);
    XSetForeground (dpy (), gc, _style->color.bgnd);
    XSetLineAttributes (dpy (), gc, 1, LineSolid, CapButt, JoinBevel);

    y = _open ? _ys - 1 : _style->step - 1;
    x = _xs - 1;
    switch (_style->type & 255)
    {
    case X_menuwin_style::BORDER:
	XDrawRectangle (dpy (), win (), gc, 0, 0, _xs - 1, y - 1);
	break;

    case X_menuwin_style::RAISED:
	XSetForeground (dpy (), gc, _style->color.shadow.lite);
	XDrawLine (dpy (), win (), gc, 0, 0, 0, y);
	XDrawLine (dpy (), win (), gc, 0, 0, x, 0);
	XSetForeground (dpy (), gc, _style->color.shadow.dark);
	XDrawLine (dpy (), win (), gc, x, y, x, 1);
	XDrawLine (dpy (), win (), gc, x, y, 1, y);
	break;
    }

    d = (_style->step + _style->font->ascent - _style->font->descent) / 2;

    XSetFont (dpy (), gc, _style->font->fid);
    for (i = 0, P = _items; i < _nit; i++, P++)
    {
	y = _ypos [i];
	XSetForeground (dpy (), gc, (P->_bits & X_menuwin_item::MASKED) ? _style->color.mask : _style->color.text);
	XDrawString (dpy (), win (), gc, DL, y + d, P->_text, _tlen [i]); 
	if (! _open) return;
	if (P->_bits & X_menuwin_item::TITLE)
	{
	    y += _style->step;
	    XDrawLine (dpy (), win (), gc, 1, y, _xs - 2, y);
	}   
    }

    if (_isel >= 0)
    {
	XSetFunction (dpy (), gc, GXxor);
	XSetForeground (dpy (), gc, _style->color.bgnd ^ _style->color.text);
	XFillRectangle (dpy (), win (), gc, DL - 2, _ypos [_isel], _xs - DL - DR + 4, _style->step);
    }
}


void X_menuwin::select (int k)
{
    if (k != _isel)
    {
        GC gc = disp ()->dgc ();
        XSetFunction (dpy (), gc, GXxor);
        XSetForeground (dpy (), gc, _style->color.bgnd ^ _style->color.text);
        if (_isel >= 0) XFillRectangle (dpy (), win (), gc, DL - 2, _ypos [_isel], _xs - DL - DR + 4, _style->step);
        _isel = k;
        if (_isel >= 0) XFillRectangle (dpy (), win (), gc, DL - 2, _ypos [_isel], _xs - DL - DR + 4, _style->step);
    }
}


int X_menuwin::findit (int x, int y)
{
    int i;

    for (i = 0; i < _nit; i++)
    {
        if (   (y > _ypos [i])
	       && (y < _ypos [i] + _style->step - 1)
	       && (! (_items [i]._bits & (X_menuwin_item::MASKED | X_menuwin_item::TITLE)))) return i;
    }
    return -1;
}


void X_menuwin::set_mask (unsigned m)
{
    for (int i = 0; i < _nit; i++)
    {
	if (m & 1) _items [i]._bits |= X_menuwin_item::MASKED;
	m >>= 1; 
    }
}


void X_menuwin::clr_mask (unsigned m)
{
    for (int i = 0; i < _nit; i++)
    {
	if (m & 1) _items [i]._bits &= ~X_menuwin_item::MASKED;
	m >>= 1; 
    }
}


