/*
    Copyright (C) 2003 Fons Adriaensen <fons.adriaensen@skynet.be>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <clxclient.h>
#include <math.h>


X_rotary::X_rotary (X_window       *parent,
                    X_callback     *callb,
                    X_rotary_style *style, 
                    int xp, int yp,
		    int rad, int rim,
                    int cbid) :
    X_window (parent, xp, yp, 2 * rad + 4, 2 * rad + 4, style->color.bgnd),
    _callb (callb),
    _style (style),
    _rad (rad),
    _rim (rim),
    _cbid (cbid)
{
    x_add_events (ExposureMask | Button1MotionMask | Button3MotionMask | ButtonPressMask | ButtonReleaseMask);
    _xc = _yc = _rad + 2;
    _ap = _am = 0;
}


X_rotary::~X_rotary (void)
{
}


void X_rotary::handle_event (XEvent *xe)
{
    switch (xe->type)
    {
    case Expose:
	expose ((XExposeEvent *) xe);
	break;  
 
    case ButtonPress:
	bpress ((XButtonEvent *) xe);
	break;  

    case ButtonRelease:
	brelease ((XButtonEvent *) xe);
	break;

    case MotionNotify:
	motion ((XMotionEvent *) xe);
	break;

    default: 
	fprintf (stderr, "rotary: event %d\n", xe->type );
    }
}


void X_rotary::bpress (XButtonEvent *e)
{
    if (_style->mode == X_rotary_style::A270) xormark ();
    _ap = atan2 (e->x - _xc, _yc - e->y) * 0.5 / M_PI;
    if (_style->mode == X_rotary_style::A270)
    {
        _am = _ap;
        if (_am >  0.375) _am = _ap =  0.375; 
	if (_am < -0.375) _am = _ap = -0.375; 
        xormark ();
	_callb->handle_callb (X_callback::ROTARY_SET, this, (XEvent *) e);
    }
}


void X_rotary::brelease (XButtonEvent *e)
{
    _callb->handle_callb (X_callback::ROTARY_STOP, this, (XEvent *) e);
}


void X_rotary::motion (XMotionEvent *e)
{
    _da = atan2 (e->x - _xc, _yc - e->y) * 0.5 / M_PI - _ap;
    while (_da >  0.5) _da -= 1;
    while (_da < -0.5) _da += 1;
    xormark ();
    _ap += _da;
    _am += _da;
    if (_style->mode != X_rotary_style::CONT)
    {
        if (_am >  0.375) _am =  0.375; 
	if (_am < -0.375) _am = -0.375; 
    }
    xormark ();
    _callb->handle_callb (X_callback::ROTARY_MOVE, this, (XEvent *) e);
}


void X_rotary::expose (XExposeEvent *e)
{
    GC gc = disp ()->dgc ();

    XSetFunction   (dpy (), gc, GXcopy);
    XSetLineAttributes (dpy (), gc, 1, LineSolid, CapButt, JoinBevel);

    if (_rim)
    {
	XSetForeground (dpy (), gc, _style->color.shadow.lite);
	XFillArc (dpy (), win (), gc, _xc - _rad, _yc - _rad, 2 * _rad, 2 * _rad, 120 * 64, 30 * 64);

	XSetForeground (dpy (), gc, _style->color.shadow.med1);
	XFillArc (dpy (), win (), gc, _xc - _rad, _yc - _rad, 2 * _rad, 2 * _rad,   0 * 64, 120 * 64);
	XFillArc (dpy (), win (), gc, _xc - _rad, _yc - _rad, 2 * _rad, 2 * _rad, 150 * 64, 120 * 64);

	XSetForeground (dpy (), gc, _style->color.shadow.dark);
	XFillArc (dpy (), win (), gc, _xc - _rad, _yc - _rad, 2 * _rad, 2 * _rad, 270 * 64, 90 * 64);

	XSetForeground (dpy (), gc, _style->color.thin);
	XDrawArc (dpy (), win (), gc, _xc - _rad, _yc - _rad, 2 * _rad, 2 * _rad,        0, 360 * 64);
    }

    int s = _rad - _rim;

    XSetForeground (dpy (), gc, _style->color.top);
    XFillArc (dpy (), win (), gc, _xc - s, _yc - s, 2 * s, 2 * s, 0, 360 * 64);

    XSetForeground (dpy (), gc, _style->color.thin);
    XDrawArc (dpy (), win (), gc, _xc - s, _yc - s, 2 * s, 2 * s, 0, 360 * 64);

    xormark ();
}


void X_rotary::xormark (void)
{
    float a, x, y, r;

    GC gc = disp ()->dgc ();

    if (_style->mark)
    {
	XSetFunction   (dpy (), gc, GXxor);
	XSetForeground (dpy (), gc, _style->color.top  ^ _style->color.mark);

	a = _am * 2 * M_PI;
	x = sin (a);
	y = cos (a);
	switch (_style->mark)
	{
	case X_rotary_style::CIRC:
	    r = _rad - _rim - 7;
	    XFillArc (dpy (), win (), gc, (int)(_xc + x * r - 3.5), (int)( _yc - y * r - 3.5), 10, 10, 0, 360 * 64);
	    break;

	case X_rotary_style::LINE:
	    XSetLineAttributes (dpy (), gc, 2, LineSolid, CapButt, JoinBevel);
	    r = _rad - _rim - 1;
	    XDrawLine (dpy (), win (), gc, (int)(_xc + 0.3 * x * r + 0.5), (int)(_yc - 0.3 * y * r + 0.5),
		       (int)(_xc + x * r + 0.5), (int)(_yc - y * r + 0.5));   
	}
    }
}


