/*
    Copyright (C) 2003 Fons Adriaensen <fons.adriaensen@skynet.be>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <clxclient.h>
#include <X11/keysym.h>


#define DX 8


X_textip::X_textip (X_window          *parent, 
                    X_callback        *callb,
                    X_textln_style    *style,
                    int xp, int yp,
                    int xs, int ys,
                    int len,
                    X_linked     *back,
                    X_linked     *forw) :
    X_window (parent, xp, yp, xs, ys, style->color.normal.bgnd),
    X_linked (back, forw),
    _callb (callb), 
    _style (style),
    _fg (_style->color.normal.text),
    _bg (_style->color.normal.bgnd),
    _flags (0),
    _xs (xs), _ys (ys),
    _max (len), _len (0), _txt (0), _hid (0),
    _x0 (xs - DX), _ic (0), _xc (_x0)
{
    x_add_events (ExposureMask);
    if (callb) x_add_events (ButtonPressMask | FocusChangeMask);
    _txt = new char [_max + 1];
    _y0 = (_ys + style->font->ascent - style->font->descent) / 2;
}


X_textip::~X_textip (void)
{
    delete [] _txt;
    delete [] _hid;
}


void X_textip::select (void)
{
    if (_callb) _callb->handle_callb (X_callback::TEXTIP_BUT, this, 0);
}


void X_textip::handle_event (XEvent *xe)
{
    switch (xe->type)
    {
    case Expose:
	expose ((XExposeEvent *) xe);
	break;  
 
    case ButtonPress:
	bpress ((XButtonEvent *) xe); 
	break;  
 
    case KeyPress:
	keypress ((XKeyEvent *) xe);
	break;

    case FocusIn:
	setfocus ((XFocusChangeEvent *) xe);
	break;

    case FocusOut:
	remfocus ((XFocusChangeEvent *) xe);
	break;

    case SelectionNotify:
	paste ((XSelectionEvent *) xe);
	break;
    }
}


void X_textip::expose (XExposeEvent *e)
{
    redraw ();
}


void X_textip::bpress (XButtonEvent *e)
{
    if (e->button == Button2)
    {
	//    if (_flacs & FOCUS) XConvertSelection (dpy (), XA_PRIMARY, XA_STRING, XInternAtom (dpy (),
	//                                 "XTIPSEL", False), win (), e->time);
    }
    else
    {
	xorcursor (); 
	_ic = findindex (e->x); 
	_xc = _x0 + textwidth (_ic);
	xorcursor ();
	select ();
    }
}


void X_textip::keypress (XKeyEvent *e)
{
    int  k;
    char t [10];

    k = XLookupString (e, t, 10, 0, 0);
    if (_flags & CLEAR) del_all ();
    if (_flags & CALLB) _callb->handle_callb (X_callback::TEXTIP_MOD, this, 0);
    _flags &= ~(CLEAR | CALLB);
    if (k && (*t >= ' ') && (*t != 127)) insert (*t);
    else spkey (XLookupKeysym (e, 0));
}


void X_textip::spkey (int k)
{
    switch (k)
    {
    case 'a':
	go_sol ();
	break;

    case XK_Left:
    case 'b':
	go_lt ();
	break;

    case XK_Delete:
    case 'd':
	del_rt ();
	break;

    case 'e':
	go_eol ();
	break;

    case XK_Right:
    case 'f':
	go_rt ();
	break;

    case 'k':
	del_eol ();
	break;

    case 'u':
	del_all ();
	break;

    case XK_BackSpace:
	del_lt ();
	break;

    case XK_Up:
	go_eol ();
	cbkey (k);
	if (_back) _back->select ();
	break;

    case XK_Tab:
    case XK_Down:
	go_eol ();
	cbkey (XK_Down);
	if (_forw) _forw->select ();
	break;

    case XK_Return:
    case XK_KP_Enter:
	go_eol ();
	cbkey (XK_Return);
	break;

    default:
	cbkey (k);
    }
}


void X_textip::cbkey (int k)
{
    _key = k; 
    if (_callb) _callb->handle_callb (X_callback::TEXTIP_KEY, this, 0);
}


void X_textip::paste (XSelectionEvent *e)
{
    int i, k;
    XTextProperty t;

    if (e->target != None)
    {
	XGetTextProperty (dpy (), win (), &t, e->property);
	k = t.nitems;
	if (k > _max - _len) XBell (dpy (), 0);
	else if (k)
	{
	    _len += k;
	    for (i = _len; i > _ic; i--) _txt [i] = _txt [i - k];
	    for (i = 0; i < k; i++) _txt [_ic++] = t.value [i];
	    _x0 = _xc - textwidth (_ic);
	    redraw ();
	}     
    }
}


void X_textip::insert (int k)
{
    if (_len < _max)
    {
	int i;

	_len++;
	for (i = _len; i > _ic; i--) _txt [i] = _txt [i-1];
	_txt [_ic++] = k;
	_x0 = _xc - textwidth (_ic);
	redraw ();
    }
    else XBell (dpy (), 0);    
}


void X_textip::go_sol (void)
{
    if (_len)
    {
	_ic = 0;
	_x0 = _xc = _xs - DX - textwidth (_len);
	if (_x0 < DX) _x0 = _xc = DX;
	redraw ();
    }
}


void X_textip::go_eol (void)
{
    if (_len)
    {
	_ic = _len;
	_xc = _xs - DX;
	_x0 = _xc - textwidth (_ic);
	redraw ();
    }
}


void X_textip::go_lt (void)
{
    if (_ic)
    {
	int d = textwidth (--_ic);   
	xorcursor ();
	_xc = _x0 + d;
	if (_xc < DX)
	{
	    _xc = DX;
	    _x0 = _xc - d;      
	    redraw ();
	}
	else xorcursor ();
    }
}


void X_textip::go_rt (void)
{
    if (_ic < _len)
    {
	int d = textwidth (++_ic);   
	xorcursor ();
	_xc = _x0 + d;
	if (_xc > _xs - DX)
	{
	    _xc = _xs - DX;
	    _x0 = _xc - d; 
	    redraw ();
	}
	else xorcursor ();
    }
}


void X_textip::del_lt (void)
{
    if (_ic)
    {
	int i, d;

	_len--;
	for (i = --_ic; i < _len; i++) _txt [i] = _txt [i+1];
	_x0 = _xc - textwidth (_ic);
	redraw ();
    }    
}


void X_textip::del_rt (void)
{
    if (_ic < _len)
    {
	int i, d;

	_len--;
	for (i = _ic; i < _len; i++) _txt [i] = _txt [i+1];
	d = _xs - DX - _x0 - textwidth (_len);
	if (d > 0)
	{
	    _x0 += d;
	    _xc += d;
	}
	redraw ();
    }    
}


void X_textip::del_eol (void)
{
    if (_ic < _len)
    {
	_len = _ic;
	_xc = _xs - DX;
	_x0 = _xc - textwidth (_ic);
	redraw ();
    }
}


void X_textip::del_all (void)
{
    if (_len)
    {
	_ic = _len = 0;
	_x0 = _xc = _xs - DX;
	redraw ();
    }
}


void X_textip::redraw (void)
{
    GC gc = disp ()->dgc ();

    XClearWindow (dpy (), win ());
    XSetFunction (dpy (), gc, GXcopy);

    if (_len)
    {
	XSetFont (dpy (), gc, _style->font->fid);
	XSetForeground (dpy (), gc, (_flags & FOCUS) ? _style->color.focus.text : _fg);
	XDrawString (dpy (), win (), gc, _x0,  _y0, _hid ? _hid : _txt, _len);
    }

    if (_callb)
    {
	XSetLineAttributes (dpy (), gc, 1, LineSolid, CapButt, JoinBevel);
	XSetForeground (dpy (), gc, _style->color.shadow.dark);
	XDrawLine (dpy (), win (), gc, 0, _ys - 2, 0, 0);
	XDrawLine (dpy (), win (), gc, 0, 0, _xs - 2, 0);
	XSetForeground (dpy (), gc, _style->color.shadow.lite);
	XDrawLine (dpy (), win (), gc, 1, _ys - 1, _xs - 1, _ys - 1);
	XDrawLine (dpy (), win (), gc, _xs - 1, _ys - 1, _xs - 1, 1);
	if (_flags * FOCUS) xorcursor ();
    }
}


void X_textip::set_text (const char *txt)
{
    _len = 0;
    if (txt) while ((*txt >= ' ') && (_len < _max)) _txt [_len++] = *txt++;
    _x0 = _xs - DX;
    _xc = _x0;
    _ic = _len;
    if (_len) _x0 -= textwidth (_len);
    redraw ();
}
   

void X_textip::set_color (unsigned long bg, unsigned long fg)
{
    _fg = fg;
    _bg = bg;
    redraw ();
}


void X_textip::setfocus (XFocusChangeEvent *e)
{
    if (e->detail != NotifyPointer && ! (_flags & FOCUS))
    {  
	_flags |= FOCUS;
	x_add_events (KeyPressMask);
	XSetWindowBackground (dpy (), win (),_style->color.focus.bgnd);
	redraw ();
    }
}


void X_textip::remfocus (XFocusChangeEvent *e)
{
    if (_flags & FOCUS)
    {  
	_flags ^= FOCUS;
	x_rem_events (KeyPressMask);
	XSetWindowBackground (dpy (), win (), _bg);
	redraw ();
    }
}


void X_textip::xorcursor (void)
{
    if (_flags & FOCUS)
    {
	GC gc = disp ()->dgc ();
	XSetForeground (dpy (), gc, _style->color.focus.bgnd ^ _style->color.focus.caret);
	XSetFunction (dpy (), gc, GXxor);
	XSetLineAttributes (dpy (), gc, 1, LineSolid, CapButt, JoinBevel);
	XDrawLine (dpy (), win (), gc, _xc, _y0 - _style->font->ascent,
		   _xc, _y0 + _style->font->descent);
    }
} 


int X_textip::findindex (int x)
{
    int i, i0, i1;

    if (x < _x0) return 0;
    i0 = 0;
    i1 = _len;
    while (i1 > i0 + 1)  
    {
	i = (i0 + i1) / 2;
	if (_x0 + textwidth (i) > x) i1 = i;
	else                         i0 = i;
    }
    return i1;   
}    


int X_textip::textwidth (int k)
{
    return XTextWidth (_style->font, _hid ? _hid : _txt, k);
}


void X_textip::hidden (int k)
{
    if (! _hid) _hid = new char [_max];
    for (int i = 0; i < _max; i++) _hid [i] = k;
}


